
#ifndef _GAMEDLL_UTILS_
#define _GAMEDLL_UTILS_

#include "sdk/amxxmodule.h"
#include "cbase_ent.h"

#ifndef GAMEDLL_MAX_PLAYERS
#define GAMEDLL_MAX_PLAYERS 32
#endif

class CPlayerHandler;

extern CPlayerHandler* g_Players;

#define GET_CPLAYER_POINTER(x)	g_Players->GetPlayer(x)
#define IS_USER_CONNECTED(x)	g_Players->IsInGame(x)
#define IS_USER_ALIVE(x)	(g_Players->IsInGame(x) & g_Players->IsAlive(x))

class CPlayer
{
public:
	edict_t* pEdict;
	cbase_t* pEntity;
	entvars_t* pev;

	int index;
	bool ingame;
	bool alive;
	int team;

	CPlayer()
	{
		pEdict = NULL;
		pEntity = NULL;
		pev = NULL;

		ingame = false;
		alive = false;
		team = 0;
	}

	void PutInServer(int id) { ingame = true; pEdict = C_EDICT(id); pEntity = C_CBASE(id); pev = C_PEV(id); };
	void Disconnect() { ingame = alive = false; team = 0; pEdict = NULL; pEntity = NULL; pev = NULL; };
	void Killed() { alive = false; };
	void Spawn() { alive = true; };
	void JoinTeam(int t) { team = t; };
	bool IsAlive() { return (ingame && pEdict->v.deadflag == DEAD_NO && pEdict->v.health > 0) ? true : false; };
	bool IsInGame() { return ingame; };
};

class CPlayerHandler
{
private:
	int maxplayers;
	CPlayer* players[GAMEDLL_MAX_PLAYERS];
public:
	CPlayerHandler(int global_maxplayers)
	{
		if(global_maxplayers > GAMEDLL_MAX_PLAYERS)
			global_maxplayers = GAMEDLL_MAX_PLAYERS;

		for(maxplayers = 0; maxplayers < global_maxplayers; maxplayers++)
		{
			players[maxplayers] = new CPlayer();
			players[maxplayers]->index = maxplayers + 1;
		}
		maxplayers++;
	}

	~CPlayerHandler()
	{
		for(maxplayers; maxplayers > 0; --maxplayers)
		{
			delete [] players[maxplayers];
		}
	}

	bool IsPlayer(int id)
	{
		if(0 < id <= maxplayers)
			return true;

		return false;
	}

	CPlayer* GetPlayer(int id) { return IsPlayer(id) ? players[id] : NULL; }
	CPlayer* GetPlayer(edict_t* e) { return GetPlayer(C_INDEX(e)); };
	CPlayer* GetPlayer(cbase_t* ent) { return GetPlayer(C_INDEX(ent)); };

	void PutInServer(int id);
	void PutInServer(edict_t* e) { PutInServer(C_INDEX(e)); };
	void PutInServer(cbase_t* ent) { PutInServer(C_INDEX(ent)); };

	void Disconnect(int id);
	void Disconnect(edict_t* e) { Disconnect(C_INDEX(e)); };
	void Disconnect(cbase_t* ent) { Disconnect(C_INDEX(ent)); };

	void Killed(int id);
	void Killed(edict_t* e) { Killed(C_INDEX(e)); };
	void Killed(cbase_t* ent) { Killed(C_INDEX(ent)); };

	void Spawn(int id);
	void Spawn(edict_t* e) { Spawn(C_INDEX(e)); };
	void Spawn(cbase_t* ent) { Spawn(C_INDEX(ent)); };

	void JoinTeam(int id, int team);
	void JoinTeam(edict_t* e, int team) { JoinTeam(C_INDEX(e), team); };
	void JoinTeam(cbase_t* ent, int team) { JoinTeam(C_INDEX(ent), team); };

	bool IsAlive(int id);
	bool IsAlive(edict_t* e) { return IsAlive(C_INDEX(e)); };
	bool IsAlive(cbase_t* ent) { return IsAlive(C_INDEX(ent)); };

	bool IsInGame(int id);
	bool IsInGame(edict_t* e) { return IsInGame(C_INDEX(e)); };
	bool IsInGame(cbase_t* ent) { return IsInGame(C_INDEX(ent)); };

};

#endif
