
#include "sdk/amxxmodule.h"
#include "zp50_core.h"

static int g_ident = 0;

int g_Forwards[FW_USER_END];

bool g_LastZombieForwardCalled = false;
bool g_LastHumanForwardCalled = false;
int g_ForwardResult;
int g_IsZombie;
int g_RespawnAsZombie;
int g_IsFirstZombie;
int g_IsLastZombie;
int g_IsLastHuman;

cvar_t cv_zp_version = {"zp_version", ZP_VERSION_STR_LONG, FCVAR_SERVER|FCVAR_SPONLY, 0, NULL};
cvar_t* zp_version;

AMX_NATIVE_INFO zp_core_natives[] = {
	 /////////////////// <--- 19 chars max in current small version
	{"zp_core_is_zombie",	zp_core_is_zombie},
	{"zp_core_is_first_zombie",	zp_core_is_first_zombie},
	{"zp_core_is_last_zombie",	zp_core_is_last_zombie},
	{"zp_core_is_last_human",	zp_core_is_last_human},
	{"zp_core_get_zombie_count",	zp_core_get_zombie_count},
	{"zp_core_get_human_count",	zp_core_get_human_count},
	{"zp_core_infect",	zp_core_infect},
	{"zp_core_cure",	zp_core_cure},
	{"zp_core_force_infect",	zp_core_force_infect},
	{"zp_core_force_cure",		zp_core_force_cure},
	{"zp_core_respawn_as_zombie",	zp_core_respawn_as_zombie},
	{NULL,	NULL}
};

void ZP_OnPluginsLoaded()
{
	g_Forwards[FW_USER_INFECT_PRE] = MF_RegisterForward("zp_fw_core_infect_pre", ET_CONTINUE, FP_CELL, FP_CELL, FP_DONE);
	g_Forwards[FW_USER_INFECT] = MF_RegisterForward("zp_fw_core_infect", ET_IGNORE, FP_CELL, FP_CELL, FP_DONE);
	g_Forwards[FW_USER_INFECT_POST] = MF_RegisterForward("zp_fw_core_infect_post", ET_IGNORE, FP_CELL, FP_CELL, FP_DONE);
	g_Forwards[FW_USER_CURE_PRE] = MF_RegisterForward("zp_fw_core_cure_pre", ET_CONTINUE, FP_CELL, FP_CELL, FP_DONE);
	g_Forwards[FW_USER_CURE] = MF_RegisterForward("zp_fw_core_cure", ET_IGNORE, FP_CELL, FP_CELL, FP_DONE);
	g_Forwards[FW_USER_CURE_POST] = MF_RegisterForward("zp_fw_core_cure_post", ET_IGNORE, FP_CELL, FP_CELL, FP_DONE);
	g_Forwards[FW_USER_LAST_ZOMBIE] = MF_RegisterForward("zp_fw_core_last_zombie", ET_IGNORE, FP_CELL, FP_DONE);
	g_Forwards[FW_USER_LAST_HUMAN] = MF_RegisterForward("zp_fw_core_last_human", ET_IGNORE, FP_CELL, FP_DONE);
	g_Forwards[FW_USER_SPAWN_POST] = MF_RegisterForward("zp_fw_core_spawn_post", ET_IGNORE, FP_CELL, FP_DONE);

	// cvar zp_version
	g_engfuncs.pfnCvar_RegisterVariable(&cv_zp_version);
	zp_version = g_engfuncs.pfnCVarGetPointer("zp_version");
}

void ZP_OnAmxxAttach()
{
	char file[ZP_MAX_STRING];
	MF_MergeDefinitionFile(MF_BuildPathnameR(file, sizeof(file) - 1, "%s/lang/zombie_plague.txt", MF_GetLocalInfo("amxx_datadir", "addons/amxmodx/data")));
	MF_MergeDefinitionFile(MF_BuildPathnameR(file, sizeof(file) - 1, "%s/lang/zombie_plague50.txt", MF_GetLocalInfo("amxx_datadir", "addons/amxmodx/data")));

	MF_AddNatives(zp_core_natives);
}

void ZP_ServerActivate_Post()
{
	char file[ZP_MAX_STRING];
	SERVER_COMMAND(MF_BuildPathnameR(file, sizeof(file) - 1, "%s/zombieplague.cfg", MF_GetLocalInfo("amxx_configsdir", "addons/amxmodx/configs")));
	RETURN_META(MRES_IGNORED);
}

void ZP_ClientDisconnect_Post(int id)
{
	flag_unset(g_IsZombie, id);
	flag_unset(g_RespawnAsZombie, id);

	CheckLastZombieHuman();
}

void ZP_PlayerSpawn_Post(int id)
{
	if(!IS_USER_ALIVE(id))
		return;

	g_ForwardResult = MF_ExecuteForward(g_Forwards[FW_USER_SPAWN_POST], id);

	if(flag_get(g_RespawnAsZombie, id))
		InfectPlayer(id, NULL);
	else
		CurePlayer(id, NULL);

	flag_unset(g_RespawnAsZombie, id);
}

void ZP_PlayerKilled_Post(int id, int attacker, int igib)
{
	CheckLastZombieHuman();
}

cell AMX_NATIVE_CALL zp_core_is_zombie(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_CONNECTED(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	return flag_get_boolean(g_IsZombie, id);
}

cell AMX_NATIVE_CALL zp_core_is_first_zombie(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_CONNECTED(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	return flag_get_boolean(g_IsFirstZombie, id);
}

cell AMX_NATIVE_CALL zp_core_is_last_zombie(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_CONNECTED(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	return flag_get_boolean(g_IsLastZombie, id);
}

cell AMX_NATIVE_CALL zp_core_is_last_human(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_CONNECTED(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	return flag_get_boolean(g_IsLastHuman, id);
}

cell AMX_NATIVE_CALL zp_core_get_zombie_count(AMX *amx, cell *params)
{
	return GetZombieCount();
}

cell AMX_NATIVE_CALL zp_core_get_human_count(AMX *amx, cell *params)
{
	return GetHumanCount();
}

cell AMX_NATIVE_CALL zp_core_infect(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_ALIVE(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	if(flag_get(g_IsZombie, id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Player already infected (%d)", id);
		return false;
	}

	InfectPlayer(id, params[2]);
	return true;
}

cell AMX_NATIVE_CALL zp_core_cure(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_ALIVE(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	if(flag_get(g_IsZombie, id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Player not infected (%d)", id);
		return false;
	}

	CurePlayer(id, params[2]);
	return true;
}

cell AMX_NATIVE_CALL zp_core_force_infect(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_ALIVE(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	InfectPlayer(id, params[2]);
	return true;
}

cell AMX_NATIVE_CALL zp_core_force_cure(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_ALIVE(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	CurePlayer(id, params[2]);
	return true;
}

cell AMX_NATIVE_CALL zp_core_respawn_as_zombie(AMX *amx, cell *params)
{
	int id = params[1];
	if(!IS_USER_CONNECTED(id))
	{
		MF_LogError(amx, AMX_ERR_NATIVE, "[ZP] Invalid Player (%d)", id);
		return false;
	}

	if(params[2])
		flag_set(g_RespawnAsZombie, id);
	else
		flag_unset(g_RespawnAsZombie, id);

	return true;
}

void CheckLastZombieHuman()
{
	int zcount = GetZombieCount();
	int hcount = GetHumanCount();
	int id, zlast, hlast;

	if(zcount == 1)
	{
		for(id = 1; id <= gpGlobals->maxClients; id++)
		{
			if(IS_USER_ALIVE(id) && flag_get(g_IsZombie, id))
			{
				flag_set(g_IsLastZombie, id);
				zlast = id;
			}
			else
			{
				flag_unset(g_IsLastZombie, id);
			}
		}
	}
	else
	{
		g_LastZombieForwardCalled = false;

		for(id = 1; id <= gpGlobals->maxClients; id++)
			flag_unset(g_IsLastZombie, id);
	}

	if(zlast > 0 && !g_LastZombieForwardCalled)
	{
		MF_ExecuteForward(g_Forwards[FW_USER_LAST_ZOMBIE], zlast);
		g_LastZombieForwardCalled = true;
	}

	if(hcount == 1)
	{
		for(id = 1; id <= gpGlobals->maxClients; id++)
		{
			if(IS_USER_ALIVE(id) && !flag_get(g_IsZombie, id))
			{
				flag_set(g_IsLastHuman, id);
				hlast = id;
			}
			else
			{
				flag_unset(g_IsLastHuman, id);
			}
		}
	}
	else
	{
		g_LastHumanForwardCalled = false;

		for(id = 1; id <= gpGlobals->maxClients; id++)
			flag_unset(g_IsLastHuman, id);
	}

	if(hlast > 0 && !g_LastHumanForwardCalled)
	{
		MF_ExecuteForward(g_Forwards[FW_USER_LAST_HUMAN], hlast);
		g_LastHumanForwardCalled = true;
	}
}

void InfectPlayer(int id, int attacker)
{
	g_ForwardResult = MF_ExecuteForward(g_Forwards[FW_USER_INFECT_PRE], id, attacker);

	if(g_ForwardResult >= PLUGIN_HANDLED)
		return;

	g_ForwardResult = MF_ExecuteForward(g_Forwards[FW_USER_INFECT], id, attacker);

	flag_set(g_IsZombie, id);

	if(GetZombieCount() == 1)
		flag_set(g_IsFirstZombie, id);
	else
		flag_unset(g_IsFirstZombie, id);

	g_ForwardResult = MF_ExecuteForward(g_Forwards[FW_USER_INFECT_POST], id, attacker);

	CheckLastZombieHuman();
}

void CurePlayer(int id, int attacker)
{
	g_ForwardResult = MF_ExecuteForward(g_Forwards[FW_USER_CURE_PRE], id, attacker);

	if(g_ForwardResult >= PLUGIN_HANDLED)
		return;

	g_ForwardResult = MF_ExecuteForward(g_Forwards[FW_USER_CURE], id, attacker);

	flag_unset(g_IsZombie, id);

	g_ForwardResult = MF_ExecuteForward(g_Forwards[FW_USER_CURE_POST], id, attacker);

	CheckLastZombieHuman();
}

int GetZombieCount()
{
	int i;
	int total = 0;

	for(i = 1; i <= gpGlobals->maxClients; i++)
	{
		if(IS_USER_ALIVE(i) && flag_get(g_IsZombie, i))
			total++;
	}
	return total;
}

int GetHumanCount()
{
	int i;
	int total = 0;

	for(i = 1; i <= gpGlobals->maxClients; i++)
	{
		if(IS_USER_ALIVE(i) && !flag_get(g_IsZombie, i))
			total++;
	}
	return total;
}

